@@ -764,10 +764,10 @@ class gpkg:
     https://www.gentoo.org/glep/glep-0078.html
     """
 
-    def __init__(self, settings, base_name=None, gpkg_file=None):
+    def __init__(self, settings, basename=None, gpkg_file=None):
         """
         gpkg class handle all gpkg operations for one package.
-        base_name is the package basename.
+        basename is the package basename.
         gpkg_file should be exists file path for read or will create.
         """
         self.settings = settings
@@ -778,9 +778,16 @@ class gpkg:
             self.gpkg_file = _unicode_decode(
                 gpkg_file, encoding=_encodings["fs"], errors="strict"
             )
-        self.base_name = base_name
+
+        if basename is None:
+            self.basename = None
+        else:
+            self.basename = basename.split("/", maxsplit=1)[-1]
+
         self.checksums = []
         self.manifest_old = []
+        self.signature_exist = None
+        self.prefix = None
 
         # Compression is the compression algorithm, if set to None will
         # not use compression.
@@ -891,7 +898,9 @@ class gpkg:
 
         # Check gpkg and metadata
         with tarfile.open(mode="r", fileobj=container_file) as container:
-            if self.gpkg_version not in container.getnames():
+            if self.gpkg_version not in (
+                os.path.basename(f) for f in container.getnames()
+            ):
                 raise InvalidBinaryPackageFormat("Invalid gpkg file.")
 
             metadata_tarinfo, metadata_comp = self._get_inner_tarinfo(
@@ -984,7 +993,7 @@ class gpkg:
         )
 
         # Long CPV
-        if len(self.base_name) >= 154:
+        if len(self.basename) >= 154:
             container_tar_format = tarfile.GNU_FORMAT
 
         # gpkg container
@@ -993,9 +1002,14 @@ class gpkg:
         )
 
         # gpkg version
-        gpkg_version_file = tarfile.TarInfo(self.gpkg_version)
+        gpkg_version_file = tarfile.TarInfo(
+            os.path.join(self.basename, self.gpkg_version)
+        )
         gpkg_version_file.mtime = datetime.utcnow().timestamp()
         container.addfile(gpkg_version_file)
+        checksum_info = checksum_helper(self.settings)
+        checksum_info.finish()
+        self._record_checksum(checksum_info, gpkg_version_file)
 
         compression_cmd = self._get_compression_cmd()
 
@@ -1058,16 +1072,19 @@ class gpkg:
                     finally:
                         image_tar.kill()
 
-    def update_metadata(self, metadata, newcpv=None):
+    def update_metadata(self, metadata, new_basename=None):
         """
         Update metadata in the gpkg file.
         """
         self._verify_binpkg()
         self.checksums = []
-        oldcpv = None
+        old_basename = self.prefix
 
-        if newcpv:
-            oldcpv = self.base_name
+        if new_basename is None:
+            new_basename = old_basename
+        else:
+            new_basename = new_basename.split("/", maxsplit=1)[-1]
+            self.basename = new_basename
 
         with open(self.gpkg_file, "rb") as container:
             container_tar_format = self._get_tar_format(container)
@@ -1080,19 +1097,19 @@ class gpkg:
             name=tmp_gpkg_file_name, mode="w", format=container_tar_format
         ) as container:
             # gpkg version
-            gpkg_version_file = tarfile.TarInfo(self.gpkg_version)
+            gpkg_version_file = tarfile.TarInfo(
+                os.path.join(new_basename, self.gpkg_version)
+            )
             gpkg_version_file.mtime = datetime.utcnow().timestamp()
             container.addfile(gpkg_version_file)
+            checksum_info = checksum_helper(self.settings)
+            checksum_info.finish()
+            self._record_checksum(checksum_info, gpkg_version_file)
 
             compression_cmd = self._get_compression_cmd()
 
             # metadata
-            if newcpv:
-                self.base_name = newcpv
-                self._add_metadata(container, metadata, compression_cmd)
-                self.base_name = oldcpv
-            else:
-                self._add_metadata(container, metadata, compression_cmd)
+            self._add_metadata(container, metadata, compression_cmd)
 
             # reuse other stuffs
             with tarfile.open(self.gpkg_file, "r") as container_old:
@@ -1100,15 +1117,17 @@ class gpkg:
 
                 for m in manifest_old:
                     file_name_old = m[1]
+                    if os.path.basename(file_name_old) == self.gpkg_version:
+                        continue
                     if os.path.basename(file_name_old).startswith("metadata"):
                         continue
-                    old_data_tarinfo = container_old.getmember(file_name_old)
+                    old_data_tarinfo = container_old.getmember(
+                        os.path.join(self.prefix, file_name_old)
+                    )
                     new_data_tarinfo = copy(old_data_tarinfo)
-                    if newcpv:
-                        m[1] = m[1].replace(oldcpv, newcpv, 1)
-                        new_data_tarinfo.name = new_data_tarinfo.name.replace(
-                            oldcpv, newcpv, 1
-                        )
+                    new_data_tarinfo.name = new_data_tarinfo.name.replace(
+                        old_basename, new_basename, 1
+                    )
                     container.addfile(
                         new_data_tarinfo, container_old.extractfile(old_data_tarinfo)
                     )
@@ -1118,6 +1137,86 @@ class gpkg:
 
         shutil.move(tmp_gpkg_file_name, self.gpkg_file)
 
+    def update_signature(self, keep_current_signature=False):
+        """
+        Add / update signature in the gpkg file.
+        if keep_current_signature is True, keep the current signature, otherwise, re-signing it.
+        """
+        self.create_signature = True
+        self._verify_binpkg()
+        self.checksums = []
+
+        with open(self.gpkg_file, "rb") as container:
+            container_tar_format = self._get_tar_format(container)
+            if container_tar_format is None:
+                raise InvalidBinaryPackageFormat("Cannot identify tar format")
+
+        # container
+        tmp_gpkg_file_name = f"{self.gpkg_file}.{os.getpid()}"
+        with tarfile.TarFile(
+            name=tmp_gpkg_file_name, mode="w", format=container_tar_format
+        ) as container:
+            # gpkg version
+            gpkg_version_file = tarfile.TarInfo(
+                os.path.join(self.prefix, self.gpkg_version)
+            )
+            gpkg_version_file.mtime = datetime.utcnow().timestamp()
+            container.addfile(gpkg_version_file)
+            checksum_info = checksum_helper(self.settings)
+            checksum_info.finish()
+            self._record_checksum(checksum_info, gpkg_version_file)
+
+            # reuse other stuffs
+            with tarfile.open(self.gpkg_file, "r") as container_old:
+                manifest_old = self.manifest_old.copy()
+                file_list_old = [f[1] for f in manifest_old]
+
+                for m in manifest_old:
+                    file_name_old = m[1]
+                    if os.path.basename(file_name_old) == self.gpkg_version:
+                        continue
+                    if os.path.basename(file_name_old).endswith(".sig"):
+                        continue
+                    old_data_tarinfo = container_old.getmember(
+                        os.path.join(self.prefix, file_name_old)
+                    )
+                    new_data_tarinfo = copy(old_data_tarinfo)
+
+                    container.addfile(
+                        new_data_tarinfo, container_old.extractfile(old_data_tarinfo)
+                    )
+                    self.checksums.append(m)
+
+                    # Check if signature file exists and reuse or create new one.
+                    if keep_current_signature and (
+                        file_name_old + ".sig" in file_list_old
+                    ):
+                        old_data_sign_tarinfo = container_old.getmember(
+                            file_name_old + ".sig"
+                        )
+                        new_data_sign_tarinfo = copy(old_data_sign_tarinfo)
+                        container.addfile(
+                            new_data_sign_tarinfo,
+                            container_old.extractfile(old_data_sign_tarinfo),
+                        )
+                        for manifest_sign in manifest_old:
+                            if manifest_sign[1] == file_name_old + ".sig":
+                                self.checksums.append(manifest_sign)
+                                break
+                    else:
+                        checksum_info = checksum_helper(
+                            self.settings, gpg_operation=checksum_helper.SIGNING
+                        )
+                        checksum_info.update(
+                            container_old.extractfile(old_data_tarinfo).read()
+                        )
+                        checksum_info.finish()
+                        self._add_signature(checksum_info, new_data_tarinfo, container)
+
+            self._add_manifest(container)
+
+        shutil.move(tmp_gpkg_file_name, self.gpkg_file)
+
     def _add_metadata(self, container, metadata, compression_cmd):
         """
         add metadata to container
@@ -1189,7 +1288,7 @@ class gpkg:
         )
 
         # Long CPV
-        if len(self.base_name) >= 154:
+        if len(self.basename) >= 154:
             container_tar_format = tarfile.GNU_FORMAT
 
         # GPKG container
@@ -1198,9 +1297,14 @@ class gpkg:
         )
 
         # GPKG version
-        gpkg_version_file = tarfile.TarInfo(self.gpkg_version)
+        gpkg_version_file = tarfile.TarInfo(
+            os.path.join(self.basename, self.gpkg_version)
+        )
         gpkg_version_file.mtime = datetime.utcnow().timestamp()
         container.addfile(gpkg_version_file)
+        checksum_info = checksum_helper(self.settings)
+        checksum_info.finish()
+        self._record_checksum(checksum_info, gpkg_version_file)
 
         compression_cmd = self._get_compression_cmd()
         # Metadata
@@ -1332,12 +1436,15 @@ class gpkg:
         Record checksum result for the given file.
         Replace old checksum if already exists.
         """
-        for c in self.checksums:
-            if c[1] == tarinfo.name:
-                self.checksums.remove(c)
-                break
 
-        checksum_record = ["DATA", tarinfo.name, str(tarinfo.size)]
+        # Remove prefix directory from the filename
+        file_name = os.path.basename(tarinfo.name)
+
+        for c in self.checksums:
+            if c[1] == file_name:
+                self.checksums.remove(c)
+
+        checksum_record = ["DATA", file_name, str(tarinfo.size)]
 
         for c in checksum_info.libs:
             checksum_record.append(c)
@@ -1364,7 +1471,14 @@ class gpkg:
             manifest.seek(0)
             manifest.write(checksum_info.gpg_output)
 
-        manifest_tarinfo = tarfile.TarInfo("Manifest")
+        if self.basename is not None:
+            basename = self.basename
+        elif self.prefix is not None:
+            basename = self.prefix
+        else:
+            raise InvalidBinaryPackageFormat("No basename or prefix specified")
+
+        manifest_tarinfo = tarfile.TarInfo(os.path.join(basename, "Manifest"))
         manifest_tarinfo.size = manifest.tell()
         manifest_tarinfo.mtime = datetime.utcnow().timestamp()
         manifest.seek(0)
@@ -1451,10 +1565,30 @@ class gpkg:
                     f"Cannot read tar file: {self.gpkg_file}"
                 )
 
-            # Check gpkg header
-            if self.gpkg_version not in container_files:
+            # Check if gpkg version file exists in any place
+            if self.gpkg_version not in (os.path.basename(f) for f in container_files):
                 raise InvalidBinaryPackageFormat(f"Invalid gpkg file: {self.gpkg_file}")
 
+            # Check how many layers are in the container
+            for f in container_files:
+                if f.startswith("/"):
+                    raise InvalidBinaryPackageFormat(
+                        f"gpkg file structure mismatch '{f}' in {self.gpkg_file}"
+                    )
+                if f.count("/") != 1:
+                    raise InvalidBinaryPackageFormat(
+                        f"gpkg file structure mismatch '{f}' in {self.gpkg_file}"
+                    )
+
+            # Check if all directories are the same in the container
+            prefix = os.path.commonpath(container_files)
+            if not prefix:
+                raise InvalidBinaryPackageFormat(
+                    f"gpkg file structure mismatch in {self.gpkg_file}, {str(container_files)}"
+                )
+
+            gpkg_version_file = os.path.join(prefix, self.gpkg_version)
+
             # If any signature exists, we assume all files have signature.
             if any(f.endswith(".sig") for f in container_files):
                 signature_exist = True
@@ -1474,13 +1608,13 @@ class gpkg:
 
             # Add all files to check list
             unverified_files = container_files.copy()
-            unverified_files.remove(self.gpkg_version)
 
             # Check Manifest file
-            if "Manifest" not in unverified_files:
+            manifest_filename = os.path.join(prefix, "Manifest")
+            if manifest_filename not in unverified_files:
                 raise MissingSignature(f"Manifest not found: {self.gpkg_file}")
 
-            manifest_file = container.extractfile("Manifest")
+            manifest_file = container.extractfile(manifest_filename)
             manifest_data = manifest_file.read()
             manifest_file.close()
 
@@ -1502,9 +1636,9 @@ class gpkg:
                         raise
 
                 manifest_data = checksum_info.gpg_output
-                unverified_files.remove("Manifest")
+                unverified_files.remove(manifest_filename)
             else:
-                unverified_files.remove("Manifest")
+                unverified_files.remove(manifest_filename)
 
             # Load manifest and create manifest check list
             manifest = self._load_manifest(manifest_data.decode("UTF-8"))
@@ -1520,7 +1654,7 @@ class gpkg:
                 # Find current file manifest record
                 manifest_record = None
                 for m in manifest:
-                    if m[1] == f:
+                    if m[1] == os.path.basename(f):
                         manifest_record = m
 
                 if manifest_record is None:
@@ -1553,6 +1687,9 @@ class gpkg:
                             gpg_operation=checksum_helper.VERIFY,
                             signature=signature,
                         )
+                    elif f == gpkg_version_file:
+                        # gpkg version file is not signed
+                        checksum_info = checksum_helper(self.settings)
                     else:
                         raise MissingSignature(
                             f"{f} signature not found in {self.gpkg_file}"
@@ -1611,6 +1748,8 @@ class gpkg:
 
         # Save current Manifest for other operations.
         self.manifest_old = manifest.copy()
+        self.signature_exist = signature_exist
+        self.prefix = prefix
 
     def _generate_metadata_from_dir(self, metadata_dir):
         """
@@ -1944,9 +2083,13 @@ class gpkg:
         else:
             raise InvalidCompressionMethod(self.compression)
 
-        data_tarinfo = tarfile.TarInfo(
-            os.path.join(self.base_name, file_name + ".tar" + ext)
-        )
+        if self.basename:
+            basename = self.basename
+        elif self.prefix:
+            basename = self.prefix
+        else:
+            raise InvalidBinaryPackageFormat("No basename or prefix specified")
+        data_tarinfo = tarfile.TarInfo(os.path.join(basename, file_name + ".tar" + ext))
         return data_tarinfo
 
     def _extract_filename_compression(self, file_name):
@@ -1973,42 +2116,23 @@ class gpkg:
         if it fail, try any file that have same name as file_name, and
         return the first one.
         """
-        if self.gpkg_version not in tar.getnames():
-            raise InvalidBinaryPackageFormat("Invalid gpkg file.")
+        if self.gpkg_version not in (os.path.basename(f) for f in tar.getnames()):
+            raise InvalidBinaryPackageFormat(f"Invalid gpkg file")
+
+        if self.basename and self.prefix and not self.prefix.startswith(self.basename):
+            writemsg(
+                colorize("WARN", f"Package basename mismatched, using {self.prefix}")
+            )
 
-        # Try get file with correct basename
-        inner_tarinfo = None
-        if self.base_name is None:
-            base_name = ""
-        else:
-            base_name = self.base_name
         all_files = tar.getmembers()
         for f in all_files:
-            if os.path.dirname(f.name) == base_name:
-                try:
-                    f_name, f_comp = self._extract_filename_compression(f.name)
-                except InvalidCompressionMethod:
-                    continue
+            try:
+                f_name, f_comp = self._extract_filename_compression(f.name)
+            except InvalidCompressionMethod:
+                continue
 
-                if f_name == file_name:
-                    return f, f_comp
-
-        # If failed, try get any file name matched
-        if inner_tarinfo is None:
-            for f in all_files:
-                try:
-                    f_name, f_comp = self._extract_filename_compression(f.name)
-                except InvalidCompressionMethod:
-                    continue
-                if f_name == file_name:
-                    if self.base_name is not None:
-                        writemsg(
-                            colorize(
-                                "WARN", "Package basename mismatched, using " + f.name
-                            )
-                        )
-                    self.base_name_alt = os.path.dirname(f.name)
-                    return f, f_comp
+            if f_name == file_name:
+                return f, f_comp
 
         # Not found
         raise FileNotFound(f"File Not found: {file_name}")
